var im = require('imagemagick');
var aws = require('aws-sdk');
var s3 = new aws.S3();
var fs = require("fs");
var async = require("async");
var CaptureScreenshotFunction = 'CaptureScreenshot'; //lambda function name

exports.handler = function(pEvent, pContext) {
    console.log('source =', pEvent.source);
    console.log('targetBucket =', pEvent.targetBucket);
    console.log('targetKey =', pEvent.targetKey);
    console.log('contentType =', pEvent.contentType);
    console.log('conversionOptions =', pEvent.conversionOptions);

    if(pEvent.source.toLowerCase().split('.').pop() == 'pdf') {
        //pdf -> download file first
        var http = require('http');
        var https = require('https');
        var protocol = (pEvent.source.split('//')[0] == 'https:' ? https : http);

        var lSrcFileName = '/tmp/'+guid()+pEvent.source.substr(pEvent.source.lastIndexOf('.'));
        var lSrcFile = fs.createWriteStream(lSrcFileName);
        protocol.get(pEvent.source, function(keyresponse) {
            keyresponse.pipe(lSrcFile);
            convert(pEvent, pContext, lSrcFileName+'[0]', function(){});
        });
    } else {
        var dlFile = '/tmp/' + guid();
        var lSteps = [];
        var pageUrl = pEvent.pageUrl || null;
        if(pageUrl){
            // take as screenshot first
            lSteps.push(function(cb){
                captureScreenshot(pEvent, pContext, cb);
            });
        }
        // download the original image/screenshot from Bucket
        lSteps.push(function(cb){
            download(pEvent, dlFile, cb);
        });
        // convert the image and upload to S3 for cache
        lSteps.push(function(cb) {
            convert(pEvent, pContext, dlFile, cb);
        });
        // Delete downloaded file
        lSteps.push(function(cb) {
            console.log('Deleting download file ==> ', dlFile);
            fs.unlink(dlFile, cb);
        });

        async.waterfall(lSteps, function (err) {
            if (err) {
                console.error(
                    ' due to an error: ' + err
                );
            } else {
                console.log(
                    'Successfully resized '
                );
            }
        });
    }
}

function captureScreenshot(pEvent, pContext, cb){
    console.log("Capturing ", pEvent.pageUrl +' '+new Date().getTime());

    async.waterfall([
        function(cb){
            var lambda = new aws.Lambda({region: process.env.AWS_REGION});

            lambda.invoke({
                FunctionName: CaptureScreenshotFunction,
                Payload: JSON.stringify(pEvent, null, 2) // pass params
            }, function(error, data) {
                if (error) {
                    console.log('error', error);
                    cb();
                }
                if(data.Payload){
                    console.log(data.Payload);
                    cb();
                }
            });
        }
    ],
    cb);
}

function guid() {
    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1);
    }
    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
        s4() + '-' + s4() + s4() + s4();
}

function uploadToS3(pFileName, pEvent, pContext, cb) {
    var lS3Params = {
        Bucket: pEvent.targetBucket,
        Key: pEvent.targetKey,
        ContentType: pEvent.contentType,
        Body: fs.readFileSync(pFileName)
    };
    
    console.log("Uploading to S3 ", pFileName, pEvent.targetKey +' '+new Date().getTime());

    s3.putObject(lS3Params, function(err, data) {
        if (err) {
            console.log(err);
            pContext.fail(err);
        }
        console.log("Succeed "+new Date().getTime());
        cb();
        pContext.succeed(pEvent.targetBucket+'/'+pEvent.targetKey);
    });
}

function convert(pEvent, pContext, pSource, cb) {
    var lSource = (pSource?pSource:pEvent.source);
    console.log("Converting from ", lSource +' '+new Date().getTime());
    var lTmpFileName = '/tmp'+pEvent.targetKey.substr(pEvent.targetKey.lastIndexOf('/'));
    console.log("Converting to ", lTmpFileName);
    var lConvertParams = [lSource];
    lConvertParams = lConvertParams.concat(pEvent.conversionOptions);
    lConvertParams.push(lTmpFileName);

    im.convert(lConvertParams,
        function(err, stdout){
            if (err) {
                console.log(err);
                pContext.fail(err);
            }
            console.log("Convert done" +' '+new Date().getTime());
            uploadToS3(lTmpFileName, pEvent, pContext, cb);
            //console.log('Uploaded ' + lTmpFileName);
        });
}

function download(pEvent, dlFile, cb) {
    //var dlStream = downloadStream(pEvent.source, pEvent.targetBucket, cb);
    console.log('downloading' +' '+new Date().getTime());
    var source= pEvent.source;
    var sourceKey = source.substr(source.indexOf(pEvent.targetBucket) + pEvent.targetBucket.length + 1);
    var dlStream = downloadStream(pEvent.targetBucket, sourceKey, cb);//'customers/a1ecf682-28cd-11e2-8049-f36004f4bcd0/images/58cb8647-5015-490d-9039-2262c6269cb9.jpg', cb);
    dlStream.on('end', function() {
        console.log('downloading done' +' '+new Date().getTime());
        cb();
    });
    dlStream.pipe(fs.createWriteStream(dlFile));
}


function downloadStream(bucket, file, cb) {
    console.log('Starting download ==> ', bucket, file);

    var req = s3.getObject({
        Bucket: bucket,
        Key: file
    });

    req.on('error', function(res) {
        req.end();
        cb('S3 download error: ' + JSON.stringify(res));
    });

    return req.createReadStream();
}